@@ -3,7 +3,7 @@
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
  *
- * Copyright 2011 Sandro Santilli <strk@keybit.net>
+ * Copyright 2011 Sandro Santilli <strk@kbt.io>
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU General Public Licence. See the COPYING file.
@@ -18,6 +18,8 @@
 #include "lwgeom_geos.h"
 #include "cu_tester.h"
 
+#include "liblwgeom_internal.h"
+
 static void test_geos_noop(void)
 {
 	int i;
@@ -64,6 +66,62 @@ static void test_geos_noop(void)
 
 }
 
+static void test_geos_linemerge(void)
+{
+	char *ewkt;
+	char *out_ewkt;
+	LWGEOM *geom1;
+	LWGEOM *geom2;
+
+	ewkt = "MULTILINESTRING((0 0, 0 100),(0 -5, 0 0))";
+	geom1 = lwgeom_from_wkt(ewkt, LW_PARSER_CHECK_NONE);
+	geom2 = lwgeom_linemerge(geom1);
+	out_ewkt = lwgeom_to_ewkt((LWGEOM*)geom2);
+	ASSERT_STRING_EQUAL(out_ewkt, "LINESTRING(0 -5,0 0,0 100)");
+	lwfree(out_ewkt);
+	lwgeom_free(geom1);
+	lwgeom_free(geom2);
+
+	ewkt = "MULTILINESTRING EMPTY";
+	geom1 = lwgeom_from_wkt(ewkt, LW_PARSER_CHECK_NONE);
+	geom2 = lwgeom_linemerge(geom1);
+	out_ewkt = lwgeom_to_ewkt((LWGEOM*)geom2);
+	ASSERT_STRING_EQUAL(out_ewkt, "GEOMETRYCOLLECTION EMPTY");
+	lwfree(out_ewkt);
+	lwgeom_free(geom1);
+	lwgeom_free(geom2);
+}
+
+
+static void test_geos_subdivide(void)
+{
+#if POSTGIS_GEOS_VERSION < 35
+	// printf("%d\n", POSTGIS_GEOS_VERSION);
+	return;
+#else
+	char *ewkt = "MULTILINESTRING((0 0, 0 100))";
+	char *out_ewkt;
+	LWGEOM *geom1 = lwgeom_from_wkt(ewkt, LW_PARSER_CHECK_NONE);
+	LWGEOM *geom2 = lwgeom_segmentize2d(geom1, 1.0);
+	
+	LWCOLLECTION *geom3 = lwgeom_subdivide(geom2, 80);
+	out_ewkt = lwgeom_to_ewkt((LWGEOM*)geom3);
+	// printf("\n--------\n%s\n--------\n", out_ewkt);
+	CU_ASSERT_EQUAL(2, geom3->ngeoms);
+	lwfree(out_ewkt);
+	lwcollection_free(geom3);
+
+	geom3 = lwgeom_subdivide(geom2, 20);
+	out_ewkt = lwgeom_to_ewkt((LWGEOM*)geom3);
+	// printf("\n--------\n%s\n--------\n", out_ewkt);
+	CU_ASSERT_EQUAL(8, geom3->ngeoms);
+	lwfree(out_ewkt);
+	lwcollection_free(geom3);
+
+	lwgeom_free(geom2);
+	lwgeom_free(geom1);
+#endif
+}
 
 /*
 ** Used by test harness to register the tests in this file.
@@ -73,4 +131,6 @@ void geos_suite_setup(void)
 {
 	CU_pSuite suite = CU_add_suite("GEOS", NULL, NULL);
 	PG_ADD_TEST(suite, test_geos_noop);
+	PG_ADD_TEST(suite, test_geos_subdivide);
+	PG_ADD_TEST(suite, test_geos_linemerge);
 }
