@@ -23,6 +23,9 @@ cu_errorreporter(const char *fmt, va_list ap);
 static void
 cu_noticereporter(const char *fmt, va_list ap);
 
+static void
+cu_debuglogger(int level, const char *fmt, va_list ap);
+
 
 /* ADD YOUR SUITE SETUP FUNCTION HERE (1 of 2) */
 extern void print_suite_setup();
@@ -38,10 +41,12 @@ extern void unionfind_suite_setup(void);
 extern void homogenize_suite_setup(void);
 extern void in_encoded_polyline_suite_setup(void);
 extern void in_geojson_suite_setup(void);
+extern void iterator_suite_setup(void);
 extern void twkb_in_suite_setup(void);
 extern void libgeom_suite_setup(void);
 extern void measures_suite_setup(void);
 extern void effectivearea_suite_setup(void);
+extern void minimum_bounding_circle_suite_setup(void);
 extern void misc_suite_setup(void);
 extern void node_suite_setup(void);
 extern void out_encoded_polyline_suite_setup(void);
@@ -52,7 +57,9 @@ extern void out_svg_suite_setup(void);
 extern void twkb_out_suite_setup(void);
 extern void out_x3d_suite_setup(void);
 extern void ptarray_suite_setup(void);
+#if HAVE_SFCGAL
 extern void sfcgal_suite_setup(void);
+#endif
 extern void split_suite_setup(void);
 extern void stringbuffer_suite_setup(void);
 extern void tree_suite_setup(void);
@@ -63,6 +70,7 @@ extern void wkb_out_suite_setup(void);
 extern void surface_suite_setup(void);
 extern void wkb_in_suite_setup(void);
 extern void wkt_in_suite_setup(void);
+extern void wrapx_suite_setup(void);
 
 
 /* AND ADD YOUR SUITE SETUP FUNCTION HERE (2 of 2) */
@@ -82,10 +90,12 @@ PG_SuiteSetup setupfuncs[] =
 #if HAVE_LIBJSON
 	in_geojson_suite_setup,
 #endif
+    iterator_suite_setup,
 	twkb_in_suite_setup,
 	libgeom_suite_setup,
 	measures_suite_setup,
 	effectivearea_suite_setup,
+	minimum_bounding_circle_suite_setup,
 	misc_suite_setup,
 	node_suite_setup,
 	out_encoded_polyline_suite_setup,
@@ -110,6 +120,7 @@ PG_SuiteSetup setupfuncs[] =
 	wkb_out_suite_setup,
 	wkt_in_suite_setup,
 	wkt_out_suite_setup,
+	wrapx_suite_setup,
 	NULL
 };
 
@@ -136,6 +147,7 @@ int main(int argc, char *argv[])
 
 	/* Install the custom error handler */
 	lwgeom_set_handlers(0, 0, 0, cu_errorreporter, cu_noticereporter);
+	lwgeom_set_debuglogger(cu_debuglogger);
 
 	/* Initialize the CUnit test registry */
 	if (CUE_SUCCESS != CU_initialize_registry())
@@ -266,7 +278,16 @@ cu_noticereporter(const char *fmt, va_list ap)
   char buf[MAX_CUNIT_MSG_LENGTH+1];
   vsnprintf (buf, MAX_CUNIT_MSG_LENGTH, fmt, ap);
   buf[MAX_CUNIT_MSG_LENGTH]='\0';
-  /*fprintf(stderr, "NOTICE: %s\n", buf);*/
+  fprintf(stderr, "NOTICE: %s\n", buf);
+}
+
+static void
+cu_debuglogger(int level, const char *fmt, va_list ap)
+{
+  char buf[MAX_CUNIT_MSG_LENGTH+1];
+  vsnprintf (buf, MAX_CUNIT_MSG_LENGTH, fmt, ap);
+  buf[MAX_CUNIT_MSG_LENGTH]='\0';
+  fprintf(stderr, "DEBUG%d: %s\n", level, buf);
 }
 
 void
@@ -274,3 +295,25 @@ cu_error_msg_reset()
 {
 	memset(cu_error_msg, '\0', MAX_CUNIT_ERROR_LENGTH);
 }
+
+/* Utility functions for testing */
+
+/* do_transformation_test
+ * - reads input_wkt and expected_wkt
+ * - asserts output of transfn(input) = expected
+ * - cleans up
+ */
+void
+do_fn_test(LWGEOM* (*transfn)(LWGEOM*), char *input_wkt, char *expected_wkt)
+{
+	LWGEOM* input = lwgeom_from_wkt(input_wkt, LW_PARSER_CHECK_NONE);
+	LWGEOM* expected = lwgeom_from_wkt(expected_wkt, LW_PARSER_CHECK_NONE);
+	LWGEOM* observed = transfn(input);
+
+	ASSERT_LWGEOM_EQUAL(observed, expected);
+
+	lwgeom_free(input);
+	lwgeom_free(expected);
+	lwgeom_free(observed);
+}
+
