@@ -3,16 +3,29 @@
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
  *
- * Copyright (C) 2011-2012 Sandro Santilli <strk@keybit.net>
+ * PostGIS is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * PostGIS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with PostGIS.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ **********************************************************************
+ *
+ * Copyright (C) 2011-2012 Sandro Santilli <strk@kbt.io>
  * Copyright (C) 2011 Paul Ramsey <pramsey@cleverelephant.ca>
  * Copyright (C) 2007-2008 Mark Cave-Ayland
  * Copyright (C) 2001-2006 Refractions Research Inc.
  *
- * This is free software; you can redistribute and/or modify it under
- * the terms of the GNU General Public Licence. See the COPYING file.
- *
  **********************************************************************/
 
+
 #ifndef _LIBLWGEOM_INTERNAL_H
 #define _LIBLWGEOM_INTERNAL_H 1
 
@@ -80,7 +93,7 @@
 #define WKB_BYTE_SIZE 1 /* Internal use only */
 
 /**
-* Well-Known Binary (WKB) Geometry Types 
+* Well-Known Binary (WKB) Geometry Types
 */
 #define WKB_POINT_TYPE 1
 #define WKB_LINESTRING_TYPE 2
@@ -197,7 +210,7 @@ LWCOLLECTION* lwcollection_simplify(const LWCOLLECTION *igeom, double dist, int
 int signum(double n);
 
 /*
-* The possible ways a pair of segments can interact. Returned by lw_segment_intersects 
+* The possible ways a pair of segments can interact. Returned by lw_segment_intersects
 */
 enum CG_SEGMENT_INTERSECTION_TYPE {
     SEG_ERROR = -1,
@@ -220,29 +233,29 @@ int lw_segment_intersects(const POINT2D *p1, const POINT2D *p2, const POINT2D *q
 double lwpoint_get_ordinate(const POINT4D *p, char ordinate);
 void lwpoint_set_ordinate(POINT4D *p, char ordinate, double value);
 
-/* 
+/*
 * Generate an interpolated coordinate p given an interpolation value and ordinate to apply it to
 */
 int point_interpolate(const POINT4D *p1, const POINT4D *p2, POINT4D *p, int hasz, int hasm, char ordinate, double interpolation_value);
 
 
 /**
-* Clip a line based on the from/to range of one of its ordinates. Use for m- and z- clipping 
+* Clip a line based on the from/to range of one of its ordinates. Use for m- and z- clipping
 */
 LWCOLLECTION *lwline_clip_to_ordinate_range(const LWLINE *line, char ordinate, double from, double to);
 
 /**
-* Clip a multi-line based on the from/to range of one of its ordinates. Use for m- and z- clipping 
+* Clip a multi-line based on the from/to range of one of its ordinates. Use for m- and z- clipping
 */
 LWCOLLECTION *lwmline_clip_to_ordinate_range(const LWMLINE *mline, char ordinate, double from, double to);
 
 /**
-* Clip a multi-point based on the from/to range of one of its ordinates. Use for m- and z- clipping 
+* Clip a multi-point based on the from/to range of one of its ordinates. Use for m- and z- clipping
 */
 LWCOLLECTION *lwmpoint_clip_to_ordinate_range(const LWMPOINT *mpoint, char ordinate, double from, double to);
 
 /**
-* Clip a point based on the from/to range of one of its ordinates. Use for m- and z- clipping 
+* Clip a point based on the from/to range of one of its ordinates. Use for m- and z- clipping
 */
 LWCOLLECTION *lwpoint_clip_to_ordinate_range(const LWPOINT *mpoint, char ordinate, double from, double to);
 
@@ -341,15 +354,15 @@ int lwcollection_startpoint(const LWCOLLECTION* col, POINT4D* pt);
  */
 void closest_point_on_segment(const POINT4D *R, const POINT4D *A, const POINT4D *B, POINT4D *ret);
 
-/* 
+/*
 * Repeated points
 */
-POINTARRAY *ptarray_remove_repeated_points_minpoints(POINTARRAY *in, double tolerance, int minpoints);
-POINTARRAY *ptarray_remove_repeated_points(POINTARRAY *in, double tolerance);
-LWGEOM* lwmpoint_remove_repeated_points(LWMPOINT *in, double tolerance);
-LWGEOM* lwline_remove_repeated_points(LWLINE *in, double tolerance);
-LWGEOM* lwcollection_remove_repeated_points(LWCOLLECTION *in, double tolerance);
-LWGEOM* lwpoly_remove_repeated_points(LWPOLY *in, double tolerance);
+POINTARRAY *ptarray_remove_repeated_points_minpoints(const POINTARRAY *in, double tolerance, int minpoints);
+POINTARRAY *ptarray_remove_repeated_points(const POINTARRAY *in, double tolerance);
+LWGEOM* lwmpoint_remove_repeated_points(const LWMPOINT *in, double tolerance);
+LWGEOM* lwline_remove_repeated_points(const LWLINE *in, double tolerance);
+LWGEOM* lwcollection_remove_repeated_points(const LWCOLLECTION *in, double tolerance);
+LWGEOM* lwpoly_remove_repeated_points(const LWPOLY *in, double tolerance);
 
 /*
 * Closure test
@@ -390,7 +403,7 @@ LWCIRCSTRING* lwcircstring_grid(const LWCIRCSTRING *line, const gridspec *grid);
 POINTARRAY* ptarray_grid(const POINTARRAY *pa, const gridspec *grid);
 
 /*
-* What side of the line formed by p1 and p2 does q fall? 
+* What side of the line formed by p1 and p2 does q fall?
 * Returns -1 for left and 1 for right and 0 for co-linearity
 */
 int lw_segment_side(const POINT2D *p1, const POINT2D *p2, const POINT2D *q);
@@ -418,7 +431,7 @@ int lwgeom_contains_point(const LWGEOM *geom, const POINT2D *pt);
 * Return 2 if the point is on the interior of the line (only case in which
 * a split happens).
 *
-* NOTE: the components pushed to the output vector have their SRID stripped 
+* NOTE: the components pushed to the output vector have their SRID stripped
 */
 int lwline_split_by_point_to(const LWLINE* ln, const LWPOINT* pt, LWMLINE* to);
 
