@@ -1,14 +1,30 @@
 /**********************************************************************
  *
  * PostGIS - Spatial Types for PostgreSQL
+ * http://postgis.net
+ *
+ * PostGIS is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * PostGIS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with PostGIS.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ **********************************************************************
  *
  * Copyright (C) 2009 Paul Ramsey <pramsey@cleverelephant.ca>
  *
- * This is free software; you can redistribute and/or modify it under
- * the terms of the GNU General Public Licence. See the COPYING file.
- *
  **********************************************************************/
 
+
+#include <math.h>
+
 #include "liblwgeom_internal.h"
 #include "lwgeom_log.h"
 
@@ -20,7 +36,7 @@ static size_t lwgeom_to_wkb_size(const LWGEOM *geom, uint8_t variant);
 */
 static char *hexchr = "0123456789ABCDEF";
 
-char* hexbytes_from_bytes(uint8_t *bytes, size_t size) 
+char* hexbytes_from_bytes(uint8_t *bytes, size_t size)
 {
 	char *hex;
 	int i;
@@ -51,7 +67,7 @@ static int lwgeom_wkb_needs_srid(const LWGEOM *geom, uint8_t variant)
 	if ( variant & WKB_NO_SRID )
 		return LW_FALSE;
 		
-	/* We can only add an SRID if the geometry has one, and the 
+	/* We can only add an SRID if the geometry has one, and the
 	   WKB form is extended */	
 	if ( (variant & WKB_EXTENDED) && lwgeom_has_srid(geom) )
 		return LW_TRUE;
@@ -274,11 +290,25 @@ static uint8_t* double_to_wkb_buf(const double d, uint8_t *buf, uint8_t variant)
 */
 static size_t empty_to_wkb_size(const LWGEOM *geom, uint8_t variant)
 {
-	size_t size = WKB_BYTE_SIZE + WKB_INT_SIZE + WKB_INT_SIZE;
+	/* endian byte + type integer */
+	size_t size = WKB_BYTE_SIZE + WKB_INT_SIZE;
 
+	/* optional srid integer */
 	if ( lwgeom_wkb_needs_srid(geom, variant) )
 		size += WKB_INT_SIZE;
 
+	/* Represent POINT EMPTY as POINT(NaN NaN) */
+	if ( geom->type == POINTTYPE )
+	{
+		const LWPOINT *pt = (LWPOINT*)geom;
+		size += WKB_DOUBLE_SIZE * FLAGS_NDIMS(pt->point->flags);		
+	}
+	/* num-elements */
+	else
+	{
+		size += WKB_INT_SIZE;
+	}
+
 	return size;
 }
 
@@ -286,13 +316,6 @@ static uint8_t* empty_to_wkb_buf(const LWGEOM *geom, uint8_t *buf, uint8_t varia
 {
 	uint32_t wkb_type = lwgeom_wkb_type(geom, variant);
 
-	if ( geom->type == POINTTYPE )
-	{
-		/* Change POINT to MULTIPOINT */
-		wkb_type &= ~WKB_POINT_TYPE;     /* clear POINT flag */
-		wkb_type |= WKB_MULTIPOINT_TYPE; /* set MULTIPOINT flag */
-	}
-
 	/* Set the endian flag */
 	buf = endian_to_wkb_buf(buf, variant);
 
@@ -303,8 +326,24 @@ static uint8_t* empty_to_wkb_buf(const LWGEOM *geom, uint8_t *buf, uint8_t varia
 	if ( lwgeom_wkb_needs_srid(geom, variant) )
 		buf = integer_to_wkb_buf(geom->srid, buf, variant);
 
-	/* Set nrings/npoints/ngeoms to zero */
-	buf = integer_to_wkb_buf(0, buf, variant);
+	/* Represent POINT EMPTY as POINT(NaN NaN) */
+	if ( geom->type == POINTTYPE )
+	{
+		const LWPOINT *pt = (LWPOINT*)geom;
+		static double nn = NAN;
+		int i;
+		for ( i = 0; i < FLAGS_NDIMS(pt->point->flags); i++ )
+		{
+			buf = double_to_wkb_buf(nn, buf, variant);
+		}
+	}
+	/* Everything else is flagged as empty using num-elements == 0 */
+	else
+	{
+		/* Set nrings/npoints/ngeoms to zero */
+		buf = integer_to_wkb_buf(0, buf, variant);
+	}
+	
 	return buf;
 }
 
@@ -346,14 +385,14 @@ static uint8_t* ptarray_to_wkb_buf(const POINTARRAY *pa, uint8_t *buf, uint8_t v
 
 	/* Bulk copy the coordinates when: dimensionality matches, output format */
 	/* is not hex, and output endian matches internal endian. */
-	if ( (dims == pa_dims) && ! wkb_swap_bytes(variant) && ! (variant & WKB_HEX)  )
+	if ( pa->npoints && (dims == pa_dims) && ! wkb_swap_bytes(variant) && ! (variant & WKB_HEX)  )
 	{
 		size_t size = pa->npoints * dims * WKB_DOUBLE_SIZE;
 		memcpy(buf, getPoint_internal(pa, 0), size);
 		buf += size;
 	}
 	/* Copy coordinates one-by-one otherwise */
-	else 
+	else
 	{
 		for ( i = 0; i < pa->npoints; i++ )
 		{
@@ -378,6 +417,10 @@ static size_t lwpoint_to_wkb_size(const LWPOINT *pt, uint8_t variant)
 	/* Endian flag + type number */
 	size_t size = WKB_BYTE_SIZE + WKB_INT_SIZE;
 
+	/* Only process empty at this level in the EXTENDED case */
+	if ( (variant & WKB_EXTENDED) && lwgeom_is_empty((LWGEOM*)pt) )
+		return empty_to_wkb_size((LWGEOM*)pt, variant);
+
 	/* Extended WKB needs space for optional SRID integer */
 	if ( lwgeom_wkb_needs_srid((LWGEOM*)pt, variant) )
 		size += WKB_INT_SIZE;
@@ -389,6 +432,10 @@ static size_t lwpoint_to_wkb_size(const LWPOINT *pt, uint8_t variant)
 
 static uint8_t* lwpoint_to_wkb_buf(const LWPOINT *pt, uint8_t *buf, uint8_t variant)
 {
+	/* Only process empty at this level in the EXTENDED case */
+	if ( (variant & WKB_EXTENDED) && lwgeom_is_empty((LWGEOM*)pt) )
+		return empty_to_wkb_buf((LWGEOM*)pt, buf, variant);
+
 	/* Set the endian flag */
 	LWDEBUGF(4, "Entering function, buf = %p", buf);
 	buf = endian_to_wkb_buf(buf, variant);
@@ -416,6 +463,10 @@ static size_t lwline_to_wkb_size(const LWLINE *line, uint8_t variant)
 	/* Endian flag + type number */
 	size_t size = WKB_BYTE_SIZE + WKB_INT_SIZE;
 
+	/* Only process empty at this level in the EXTENDED case */
+	if ( (variant & WKB_EXTENDED) && lwgeom_is_empty((LWGEOM*)line) )
+		return empty_to_wkb_size((LWGEOM*)line, variant);
+
 	/* Extended WKB needs space for optional SRID integer */
 	if ( lwgeom_wkb_needs_srid((LWGEOM*)line, variant) )
 		size += WKB_INT_SIZE;
@@ -427,6 +478,10 @@ static size_t lwline_to_wkb_size(const LWLINE *line, uint8_t variant)
 
 static uint8_t* lwline_to_wkb_buf(const LWLINE *line, uint8_t *buf, uint8_t variant)
 {
+	/* Only process empty at this level in the EXTENDED case */
+	if ( (variant & WKB_EXTENDED) && lwgeom_is_empty((LWGEOM*)line) )
+		return empty_to_wkb_buf((LWGEOM*)line, buf, variant);
+
 	/* Set the endian flag */
 	buf = endian_to_wkb_buf(buf, variant);
 	/* Set the geometry type */
@@ -447,6 +502,10 @@ static size_t lwtriangle_to_wkb_size(const LWTRIANGLE *tri, uint8_t variant)
 	/* endian flag + type number + number of rings */
 	size_t size = WKB_BYTE_SIZE + WKB_INT_SIZE + WKB_INT_SIZE;
 
+	/* Only process empty at this level in the EXTENDED case */
+	if ( (variant & WKB_EXTENDED) && lwgeom_is_empty((LWGEOM*)tri) )
+		return empty_to_wkb_size((LWGEOM*)tri, variant);
+
 	/* Extended WKB needs space for optional SRID integer */
 	if ( lwgeom_wkb_needs_srid((LWGEOM*)tri, variant) )
 		size += WKB_INT_SIZE;
@@ -459,6 +518,10 @@ static size_t lwtriangle_to_wkb_size(const LWTRIANGLE *tri, uint8_t variant)
 
 static uint8_t* lwtriangle_to_wkb_buf(const LWTRIANGLE *tri, uint8_t *buf, uint8_t variant)
 {
+	/* Only process empty at this level in the EXTENDED case */
+	if ( (variant & WKB_EXTENDED) && lwgeom_is_empty((LWGEOM*)tri) )
+		return empty_to_wkb_buf((LWGEOM*)tri, buf, variant);
+
 	/* Set the endian flag */
 	buf = endian_to_wkb_buf(buf, variant);
 	
@@ -486,6 +549,10 @@ static size_t lwpoly_to_wkb_size(const LWPOLY *poly, uint8_t variant)
 	/* endian flag + type number + number of rings */
 	size_t size = WKB_BYTE_SIZE + WKB_INT_SIZE + WKB_INT_SIZE;
 	int i = 0;
+	
+	/* Only process empty at this level in the EXTENDED case */
+	if ( (variant & WKB_EXTENDED) && lwgeom_is_empty((LWGEOM*)poly) )
+		return empty_to_wkb_size((LWGEOM*)poly, variant);
 
 	/* Extended WKB needs space for optional SRID integer */
 	if ( lwgeom_wkb_needs_srid((LWGEOM*)poly, variant) )
@@ -504,6 +571,10 @@ static uint8_t* lwpoly_to_wkb_buf(const LWPOLY *poly, uint8_t *buf, uint8_t vari
 {
 	int i;
 
+	/* Only process empty at this level in the EXTENDED case */
+	if ( (variant & WKB_EXTENDED) && lwgeom_is_empty((LWGEOM*)poly) )
+		return empty_to_wkb_buf((LWGEOM*)poly, buf, variant);
+
 	/* Set the endian flag */
 	buf = endian_to_wkb_buf(buf, variant);
 	/* Set the geometry type */
@@ -525,7 +596,7 @@ static uint8_t* lwpoly_to_wkb_buf(const LWPOLY *poly, uint8_t *buf, uint8_t vari
 
 /*
 * MULTIPOINT, MULTILINESTRING, MULTIPOLYGON, GEOMETRYCOLLECTION
-* MULTICURVE, COMPOUNDCURVE, MULTISURFACE, CURVEPOLYGON, TIN, 
+* MULTICURVE, COMPOUNDCURVE, MULTISURFACE, CURVEPOLYGON, TIN,
 * POLYHEDRALSURFACE
 */
 static size_t lwcollection_to_wkb_size(const LWCOLLECTION *col, uint8_t variant)
@@ -582,7 +653,7 @@ static size_t lwgeom_to_wkb_size(const LWGEOM *geom, uint8_t variant)
 		return 0;
 
 	/* Short circuit out empty geometries */
-	if ( lwgeom_is_empty(geom) )
+	if ( (!(variant & WKB_EXTENDED)) && lwgeom_is_empty(geom) )
 	{
 		return empty_to_wkb_size(geom, variant);
 	}
@@ -636,7 +707,8 @@ static size_t lwgeom_to_wkb_size(const LWGEOM *geom, uint8_t variant)
 static uint8_t* lwgeom_to_wkb_buf(const LWGEOM *geom, uint8_t *buf, uint8_t variant)
 {
 
-	if ( lwgeom_is_empty(geom) )
+	/* Do not simplify empties when outputting to canonical form */
+	if ( lwgeom_is_empty(geom) & ! (variant & WKB_EXTENDED) )
 		return empty_to_wkb_buf(geom, buf, variant);
 
 	switch ( geom->type )
@@ -727,7 +799,7 @@ uint8_t* lwgeom_to_wkb(const LWGEOM *geom, uint8_t variant, size_t *size_out)
 	if ( ! (variant & WKB_NDR || variant & WKB_XDR) ||
 	       (variant & WKB_NDR && variant & WKB_XDR) )
 	{
-		if ( getMachineEndian() == NDR ) 
+		if ( getMachineEndian() == NDR )
 			variant = variant | WKB_NDR;
 		else
 			variant = variant | WKB_XDR;
