@@ -1,10 +1,33 @@
-
 /**********************************************************************
- * $Id: measures.c 5439 2010-03-16 03:13:33Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
- * Copyright 2011 Nicklas Avn
+ *
+ * PostGIS is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * PostGIS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with PostGIS.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ **********************************************************************
+ *
+ * Copyright 2011 Nicklas Avén
+ *
+ **********************************************************************/
+
+
+/**********************************************************************
+ *
+ * PostGIS - Spatial Types for PostgreSQL
+ * http://postgis.net
+ * Copyright 2011 Nicklas Avén
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU General Public Licence. See the COPYING file.
@@ -18,19 +41,77 @@
 #include "lwgeom_log.h"
 
 
-/*------------------------------------------------------------------------------------------------------------
-Initializing functions
-The functions starting the distance-calculation processses
---------------------------------------------------------------------------------------------------------------*/
+static inline int
+get_3dvector_from_points(POINT3DZ *p1,POINT3DZ *p2, VECTOR3D *v)
+{
+	v->x=p2->x-p1->x;
+	v->y=p2->y-p1->y;
+	v->z=p2->z-p1->z;
+	
+	return LW_TRUE;
+}
+
+static inline int
+get_3dcross_product(VECTOR3D *v1,VECTOR3D *v2, VECTOR3D *v)
+{
+	v->x=(v1->y*v2->z)-(v1->z*v2->y);
+	v->y=(v1->z*v2->x)-(v1->x*v2->z);
+	v->z=(v1->x*v2->y)-(v1->y*v2->x);
+
+	return LW_TRUE;
+}
+
+
+/**
+This function is used to create a vertical line used for cases where one if the
+geometries lacks z-values. The vertical line crosses the 2d point that is closest
+and the z-range is from maxz to minz in the geoemtrie that has z values.
+*/
+static
+LWGEOM* create_v_line(const LWGEOM *lwgeom,double x, double y, int srid)
+{
+	
+	LWPOINT *lwpoints[2];
+	GBOX gbox;
+	int rv = lwgeom_calculate_gbox(lwgeom, &gbox);
+	
+	if ( rv == LW_FAILURE )
+		return NULL;
+	
+	lwpoints[0] = lwpoint_make3dz(srid, x, y, gbox.zmin);
+	lwpoints[1] = lwpoint_make3dz(srid, x, y, gbox.zmax);
+	
+	 return (LWGEOM *)lwline_from_ptarray(srid, 2, lwpoints);		
+}
+
+LWGEOM *
+lwgeom_closest_line_3d(const LWGEOM *lw1, const LWGEOM *lw2)
+{
+	return lw_dist3d_distanceline(lw1, lw2, lw1->srid, DIST_MIN);
+}
+
+LWGEOM *
+lwgeom_furthest_line_3d(LWGEOM *lw1, LWGEOM *lw2)
+{
+	return lw_dist3d_distanceline(lw1, lw2, lw1->srid, DIST_MAX);
+}
+
+LWGEOM *
+lwgeom_closest_point_3d(const LWGEOM *lw1, const LWGEOM *lw2)
+{
+	return lw_dist3d_distancepoint(lw1, lw2, lw1->srid, DIST_MIN);
+}
+
 
 /**
 Function initializing 3dshortestline and 3dlongestline calculations.
 */
 LWGEOM *
-lw_dist3d_distanceline(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode)
+lw_dist3d_distanceline(const LWGEOM *lw1, const LWGEOM *lw2, int srid, int mode)
 {
-	double x1,x2,y1,y2, z1, z2;
-	double initdistance = ( mode == DIST_MIN ? MAXFLOAT : -1.0);
+	LWDEBUG(2, "lw_dist3d_distanceline is called");
+	double x1,x2,y1,y2, z1, z2, x, y;
+	double initdistance = ( mode == DIST_MIN ? FLT_MAX : -1.0);
 	DISTPTS3D thedl;
 	LWPOINT *lwpoints[2];
 	LWGEOM *result;
@@ -39,14 +120,69 @@ lw_dist3d_distanceline(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode)
 	thedl.distance = initdistance;
 	thedl.tolerance = 0.0;
 
-	LWDEBUG(2, "lw_dist3d_distanceline is called");
-	if (!lw_dist3d_recursive(lw1, lw2, &thedl))
+	/*Check if we really have 3D geoemtries*/
+	/*If not, send it to 2D-calculations which will give the same result*/
+	/*as an infinite z-value at one or two of the geometries*/
+	if(!lwgeom_has_z(lw1) || !lwgeom_has_z(lw2))
 	{
-		/*should never get here. all cases ought to be error handled earlier*/
-		lwerror("Some unspecified error.");
-		result = (LWGEOM *)lwcollection_construct_empty(COLLECTIONTYPE, srid, 0, 0);
-	}
+		
+		lwnotice("One or both of the geometries is missing z-value. The unknown z-value will be regarded as \"any value\"");
+		
+		if(!lwgeom_has_z(lw1) && !lwgeom_has_z(lw2))
+			return lw_dist2d_distanceline(lw1, lw2, srid, mode);	
+		
+		DISTPTS thedl2d;
+		thedl2d.mode = mode;
+		thedl2d.distance = initdistance;
+		thedl2d.tolerance = 0.0;
+		if (!lw_dist2d_comp( lw1,lw2,&thedl2d))
+		{
+			/*should never get here. all cases ought to be error handled earlier*/
+			lwerror("Some unspecified error.");
+			result = (LWGEOM *)lwcollection_construct_empty(COLLECTIONTYPE, srid, 0, 0);
+		}
+		LWGEOM *vertical_line;
+		if(!lwgeom_has_z(lw1))
+		{
+			x=thedl2d.p1.x;
+			y=thedl2d.p1.y;
 
+			vertical_line = create_v_line(lw2,x,y,srid);
+			if (!lw_dist3d_recursive(vertical_line, lw2, &thedl))
+			{
+				/*should never get here. all cases ought to be error handled earlier*/
+				lwfree(vertical_line);
+				lwerror("Some unspecified error.");
+				result = (LWGEOM *)lwcollection_construct_empty(COLLECTIONTYPE, srid, 0, 0);
+			}			
+			lwfree(vertical_line);	
+		}	
+		if(!lwgeom_has_z(lw2))
+		{
+			x=thedl2d.p2.x;
+			y=thedl2d.p2.y;			
+			
+			vertical_line = create_v_line(lw1,x,y,srid);
+			if (!lw_dist3d_recursive(lw1, vertical_line, &thedl))
+			{
+				/*should never get here. all cases ought to be error handled earlier*/
+				lwfree(vertical_line);
+				lwerror("Some unspecified error.");
+				return (LWGEOM *)lwcollection_construct_empty(COLLECTIONTYPE, srid, 0, 0);
+			}	
+			lwfree(vertical_line);		
+		}			
+				
+	}
+	else
+	{		
+		if (!lw_dist3d_recursive(lw1, lw2, &thedl))
+		{
+			/*should never get here. all cases ought to be error handled earlier*/
+			lwerror("Some unspecified error.");
+			result = (LWGEOM *)lwcollection_construct_empty(COLLECTIONTYPE, srid, 0, 0);
+		}
+	}
 	/*if thedl.distance is unchanged there where only empty geometries input*/
 	if (thedl.distance == initdistance)
 	{
@@ -62,7 +198,6 @@ lw_dist3d_distanceline(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode)
 		y2=thedl.p2.y;
 		z2=thedl.p2.z;
 
-
 		lwpoints[0] = lwpoint_make3dz(srid, x1, y1, z1);
 		lwpoints[1] = lwpoint_make3dz(srid, x2, y2, z2);
 
@@ -76,11 +211,12 @@ lw_dist3d_distanceline(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode)
 Function initializing 3dclosestpoint calculations.
 */
 LWGEOM *
-lw_dist3d_distancepoint(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode)
+lw_dist3d_distancepoint(const LWGEOM *lw1, const LWGEOM *lw2, int srid, int mode)
 {
+	
 	double x,y,z;
 	DISTPTS3D thedl;
-	double initdistance = MAXFLOAT;
+	double initdistance = FLT_MAX;
 	LWGEOM *result;
 
 	thedl.mode = mode;
@@ -88,14 +224,72 @@ lw_dist3d_distancepoint(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode)
 	thedl.tolerance = 0;
 
 	LWDEBUG(2, "lw_dist3d_distancepoint is called");
+	
+	/*Check if we really have 3D geoemtries*/
+	/*If not, send it to 2D-calculations which will give the same result*/
+	/*as an infinite z-value at one or two of the geometries*/
+	if(!lwgeom_has_z(lw1) || !lwgeom_has_z(lw2))
+	{		
+		lwnotice("One or both of the geometries is missing z-value. The unknown z-value will be regarded as \"any value\"");
+				
+		if(!lwgeom_has_z(lw1) && !lwgeom_has_z(lw2))
+			return lw_dist2d_distancepoint(lw1, lw2, srid, mode);
+			
+		
+		DISTPTS thedl2d;
+		thedl2d.mode = mode;
+		thedl2d.distance = initdistance;
+		thedl2d.tolerance = 0.0;
+		if (!lw_dist2d_comp( lw1,lw2,&thedl2d))
+		{
+			/*should never get here. all cases ought to be error handled earlier*/
+			lwerror("Some unspecified error.");
+			return (LWGEOM *)lwcollection_construct_empty(COLLECTIONTYPE, srid, 0, 0);
+		}
+		
+		LWGEOM *vertical_line;
+		if(!lwgeom_has_z(lw1))
+		{
+			x=thedl2d.p1.x;
+			y=thedl2d.p1.y;
+			
+			vertical_line = create_v_line(lw2,x,y,srid);	
+			if (!lw_dist3d_recursive(vertical_line, lw2, &thedl))
+			{
+				/*should never get here. all cases ought to be error handled earlier*/
+				lwfree(vertical_line);	
+				lwerror("Some unspecified error.");
+				return (LWGEOM *)lwcollection_construct_empty(COLLECTIONTYPE, srid, 0, 0);
+			}		
+			lwfree(vertical_line);		
+		}	
+				
+		if(!lwgeom_has_z(lw2))
+		{
+			x=thedl2d.p2.x;
+			y=thedl2d.p2.y;
 
-	if (!lw_dist3d_recursive(lw1, lw2, &thedl))
+			vertical_line = create_v_line(lw1,x,y,srid);
+			if (!lw_dist3d_recursive(lw1, vertical_line, &thedl))
+			{
+				/*should never get here. all cases ought to be error handled earlier*/
+				lwfree(vertical_line);	
+				lwerror("Some unspecified error.");
+				result = (LWGEOM *)lwcollection_construct_empty(COLLECTIONTYPE, srid, 0, 0);
+			}	
+			lwfree(vertical_line);			
+		}	
+		
+	}
+	else
 	{
-		/*should never get here. all cases ought to be error handled earlier*/
-		lwerror("Some unspecified error.");
-		result = (LWGEOM *)lwcollection_construct_empty(COLLECTIONTYPE, srid, 0, 0);
+		if (!lw_dist3d_recursive(lw1, lw2, &thedl))
+		{
+			/*should never get here. all cases ought to be error handled earlier*/
+			lwerror("Some unspecified error.");
+			result = (LWGEOM *)lwcollection_construct_empty(COLLECTIONTYPE, srid, 0, 0);
+		}
 	}
-
 	if (thedl.distance == initdistance)
 	{
 		LWDEBUG(3, "didn't find geometries to measure between, returning null");
@@ -114,10 +308,10 @@ lw_dist3d_distancepoint(LWGEOM *lw1, LWGEOM *lw2,int srid,int mode)
 
 
 /**
-Function initialazing 3d max distance calculation
+Function initializing 3d max distance calculation
 */
 double
-lwgeom_maxdistance3d(LWGEOM *lw1, LWGEOM *lw2)
+lwgeom_maxdistance3d(const LWGEOM *lw1, const LWGEOM *lw2)
 {
 	LWDEBUG(2, "lwgeom_maxdistance3d is called");
 
@@ -125,12 +319,17 @@ lwgeom_maxdistance3d(LWGEOM *lw1, LWGEOM *lw2)
 }
 
 /**
-Function handling 3d max distance calculations and dfyllywithin calculations.
+Function handling 3d max distance calculations and dfullywithin calculations.
 The difference is just the tolerance.
 */
 double
-lwgeom_maxdistance3d_tolerance(LWGEOM *lw1, LWGEOM *lw2, double tolerance)
+lwgeom_maxdistance3d_tolerance(const LWGEOM *lw1, const LWGEOM *lw2, double tolerance)
 {
+	if(!lwgeom_has_z(lw1) || !lwgeom_has_z(lw2))
+	{
+		lwnotice("One or both of the geometries is missing z-value. The unknown z-value will be regarded as \"any value\"");
+		return lwgeom_maxdistance2d_tolerance(lw1, lw2, tolerance);	
+	}
 	/*double thedist;*/
 	DISTPTS3D thedl;
 	LWDEBUG(2, "lwgeom_maxdistance3d_tolerance is called");
@@ -147,10 +346,10 @@ lwgeom_maxdistance3d_tolerance(LWGEOM *lw1, LWGEOM *lw2, double tolerance)
 }
 
 /**
-	Function initialazing 3d min distance calculation
+	Function initializing 3d min distance calculation
 */
 double
-lwgeom_mindistance3d(LWGEOM *lw1, LWGEOM *lw2)
+lwgeom_mindistance3d(const LWGEOM *lw1, const LWGEOM *lw2)
 {
 	LWDEBUG(2, "lwgeom_mindistance3d is called");
 	return lwgeom_mindistance3d_tolerance( lw1, lw2, 0.0 );
@@ -161,12 +360,18 @@ lwgeom_mindistance3d(LWGEOM *lw1, LWGEOM *lw2)
 	The difference is just the tolerance.
 */
 double
-lwgeom_mindistance3d_tolerance(LWGEOM *lw1, LWGEOM *lw2, double tolerance)
+lwgeom_mindistance3d_tolerance(const LWGEOM *lw1, const LWGEOM *lw2, double tolerance)
 {
+	if(!lwgeom_has_z(lw1) || !lwgeom_has_z(lw2))
+	{
+		lwnotice("One or both of the geometries is missing z-value. The unknown z-value will be regarded as \"any value\"");
+		
+		return lwgeom_mindistance2d_tolerance(lw1, lw2, tolerance);	
+	}
 	DISTPTS3D thedl;
 	LWDEBUG(2, "lwgeom_mindistance3d_tolerance is called");
 	thedl.mode = DIST_MIN;
-	thedl.distance= MAXFLOAT;
+	thedl.distance= FLT_MAX;
 	thedl.tolerance = tolerance;
 	if (lw_dist3d_recursive(lw1, lw2, &thedl))
 	{
@@ -174,7 +379,7 @@ lwgeom_mindistance3d_tolerance(LWGEOM *lw1, LWGEOM *lw2, double tolerance)
 	}
 	/*should never get here. all cases ought to be error handled earlier*/
 	lwerror("Some unspecified error.");
-	return MAXFLOAT;
+	return FLT_MAX;
 }
 
 
@@ -189,7 +394,7 @@ Functions preparing geometries for distance-calculations
 
 
 /**
-This is a recursive function delivering every possible combinatin of subgeometries
+This is a recursive function delivering every possible combination of subgeometries
 */
 int lw_dist3d_recursive(const LWGEOM *lwg1,const LWGEOM *lwg2, DISTPTS3D *dl)
 {
@@ -269,10 +474,10 @@ int lw_dist3d_recursive(const LWGEOM *lwg1,const LWGEOM *lwg2, DISTPTS3D *dl)
 
 /**
 
-This function distributes the brut-force for 3D so far the only type, tasks depending on type
+This function distributes the brute-force for 3D so far the only type, tasks depending on type
 */
 int
-lw_dist3d_distribute_bruteforce(LWGEOM *lwg1, LWGEOM *lwg2, DISTPTS3D *dl)
+lw_dist3d_distribute_bruteforce(const LWGEOM *lwg1, const LWGEOM *lwg2, DISTPTS3D *dl)
 {
 
 	int	t1 = lwg1->type;
@@ -406,8 +611,8 @@ lw_dist3d_point_line(LWPOINT *point, LWLINE *line, DISTPTS3D *dl)
 
 Computes point to polygon distance
 For mindistance that means:
-1)find the plane of the polygon 
-2)projecting the point to the plane of the polygon 
+1)find the plane of the polygon
+2)projecting the point to the plane of the polygon
 3)finding if that projected point is inside the polygon, if so the distance is measured to that projected point
 4) if not in polygon above, check the distance against the boundary of the polygon
 for max distance it is always point against boundary
@@ -489,7 +694,7 @@ int lw_dist3d_poly_poly(LWPOLY *poly1, LWPOLY *poly2, DISTPTS3D *dl)
 	if(!define_plane(poly2->rings[0], &plane))
 		return LW_FALSE;
 	
-	/*What we do here is to compare the bondary of one polygon with the other polygon 
+	/*What we do here is to compare the bondary of one polygon with the other polygon
 	and then take the second boudary comparing with the first polygon*/
 	dl->twisted=1;
 	if(!lw_dist3d_ptarray_poly(poly1->rings[0], poly2,&plane, dl))
@@ -586,6 +791,15 @@ lw_dist3d_pt_seg(POINT3DZ *p, POINT3DZ *A, POINT3DZ *B, DISTPTS3D *dl)
 	return lw_dist3d_pt_pt(p,&c,dl);
 }
 
+double
+distance3d_pt_pt(const POINT3D *p1, const POINT3D *p2)
+{
+  double dx = p2->x - p1->x;
+  double dy = p2->y - p1->y;
+  double dz = p2->z - p1->z;
+  return sqrt ( dx*dx + dy*dy + dz*dz);
+}
+
 
 /**
 
@@ -680,7 +894,7 @@ lw_dist3d_ptarray_ptarray(POINTARRAY *l1, POINTARRAY *l2,DISTPTS3D *dl)
 
 Finds the two closest points on two linesegments
 */
-int 
+int
 lw_dist3d_seg_seg(POINT3DZ *s1p1, POINT3DZ *s1p2, POINT3DZ *s2p1, POINT3DZ *s2p2, DISTPTS3D *dl)
 {
 	VECTOR3D v1, v2, vl;
@@ -689,12 +903,12 @@ lw_dist3d_seg_seg(POINT3DZ *s1p1, POINT3DZ *s1p2, POINT3DZ *s2p1, POINT3DZ *s2p2
 	double a, b, c, d, e, D;
 			
 	/*s1p1 and s1p2 are the same point */
-	if (  ( s1p1->x == s1p2->x) && (s1p1->y == s1p2->y) && (s1p1->z == s1p2->y) )
+	if (  ( s1p1->x == s1p2->x) && (s1p1->y == s1p2->y) && (s1p1->z == s1p2->z) )
 	{
 		return lw_dist3d_pt_seg(s1p1,s2p1,s2p2,dl);
 	}
 	/*s2p1 and s2p2 are the same point */
-	if (  ( s2p1->x == s2p2->x) && (s2p1->y == s2p2->y) && (s2p1->z == s2p2->y) )
+	if (  ( s2p1->x == s2p2->x) && (s2p1->y == s2p2->y) && (s2p1->z == s2p2->z) )
 	{
 		dl->twisted= ((dl->twisted) * (-1));
 		return lw_dist3d_pt_seg(s2p1,s1p1,s1p2,dl);
@@ -720,10 +934,10 @@ lw_dist3d_seg_seg(POINT3DZ *s1p1, POINT3DZ *s1p2, POINT3DZ *s2p1, POINT3DZ *s2p2
 	c = DOT(v2,v2);
 	d = DOT(v1,vl);
 	e = DOT(v2,vl);
-	D = a*c - b*b; 
+	D = a*c - b*b;
 
 
-	if (D <0.000000001) 
+	if (D <0.000000001)
 	{        /* the lines are almost parallel*/
 		s1k = 0.0; /*If the lines are paralell we try by using the startpoint of first segment. If that gives a projected point on the second line outside segment 2 it wil be found that s2k is >1 or <0.*/
 		if(b>c)   /* use the largest denominator*/
@@ -735,7 +949,7 @@ lw_dist3d_seg_seg(POINT3DZ *s1p1, POINT3DZ *s1p2, POINT3DZ *s2p1, POINT3DZ *s2p2
 			s2k =e/c;
 		}
 	}
-	else 
+	else
 	{
 		s1k = (b*e - c*d) / D;
 		s2k = (a*e - b*d) / D;
@@ -744,7 +958,7 @@ lw_dist3d_seg_seg(POINT3DZ *s1p1, POINT3DZ *s1p2, POINT3DZ *s2p1, POINT3DZ *s2p2
 	/* Now we check if the projected closest point on the infinite lines is outside our segments. If so the combinations with start and end points will be tested*/
 	if(s1k<0.0||s1k>1.0||s2k<0.0||s2k>1.0)
 	{
-		if(s1k<0.0) 
+		if(s1k<0.0)
 		{
 
 			if (!lw_dist3d_pt_seg(s1p1, s2p1, s2p2, dl))
@@ -797,7 +1011,7 @@ lw_dist3d_seg_seg(POINT3DZ *s1p1, POINT3DZ *s1p2, POINT3DZ *s2p1, POINT3DZ *s2p2
 
 /**
 
-Checking if the point projected on the plane of the polygon actually is inside that polygon. 
+Checking if the point projected on the plane of the polygon actually is inside that polygon.
 If so the mindistance is between that projected point and our original point.
 If not we check from original point to the bounadary.
 If the projected point is inside a hole of the polygon we check the distance to the boudary of that hole.
@@ -860,7 +1074,7 @@ int lw_dist3d_ptarray_poly(POINTARRAY *pa, LWPOLY *poly,PLANE3D *plane, DISTPTS3
 		
 		/*If s1and s2 has different signs that means they are on different sides of the plane of the polygon.
 		That means that the edge between the points crosses the plane and might intersect with the polygon*/
-		if((s1*s2)<=0) 
+		if((s1*s2)<=0)
 		{
 			f=fabs(s1)/(fabs(s1)+fabs(s2)); /*The size of s1 and s2 is the distance from the point to the plane.*/
 			get_3dvector_from_points(&projp1, &projp2,&projp1_projp2);
@@ -883,7 +1097,7 @@ int lw_dist3d_ptarray_poly(POINTARRAY *pa, LWPOLY *poly,PLANE3D *plane, DISTPTS3
 						break;
 					}
 				}		
-				if(intersects) 
+				if(intersects)
 				{
 					dl->distance=0.0;
 					dl->p1.x=intersectionp.x;
@@ -985,7 +1199,7 @@ define_plane(POINTARRAY *pa, PLANE3D *pl)
 
 Finds a point on a plane from where the original point is perpendicular to the plane
 */
-double 
+double
 project_point_on_plane(POINT3DZ *p,  PLANE3D *pl, POINT3DZ *p0)
 {
 /*In our plane definition we have a point on the plane and a normal vektor (pl.pv), perpendicular to the plane
@@ -1003,7 +1217,7 @@ So, we already have a direction from p to find p0, but we don't know the distanc
 	
 	p0->x=p->x+pl->pv.x*f;
 	p0->y=p->y+pl->pv.y*f;
-	p0->z=p->z+pl->pv.z*f;      
+	p0->z=p->z+pl->pv.z*f;
 	
 	return f;		
 }
