@@ -92,7 +92,13 @@ static void PROJ4SRSCacheInit(MemoryContext context);
 static void PROJ4SRSCacheDelete(MemoryContext context);
 static void PROJ4SRSCacheReset(MemoryContext context);
 static bool PROJ4SRSCacheIsEmpty(MemoryContext context);
+
+#if POSTGIS_PGSQL_VERSION >= 96
+static void PROJ4SRSCacheStats(MemoryContext context, int level, bool print, MemoryContextCounters *totals);
+#else
 static void PROJ4SRSCacheStats(MemoryContext context, int level);
+#endif
+
 #ifdef MEMORY_CONTEXT_CHECKING
 static void PROJ4SRSCacheCheck(MemoryContext context);
 #endif
@@ -165,7 +171,11 @@ PROJ4SRSCacheIsEmpty(MemoryContext context)
 }
 
 static void
+#if POSTGIS_PGSQL_VERSION >= 96
+PROJ4SRSCacheStats(MemoryContext context, int level, bool print, MemoryContextCounters *totals)
+#else
 PROJ4SRSCacheStats(MemoryContext context, int level)
+#endif
 {
 	/*
 	 * Simple stats display function - we must supply a function since this call is mandatory according to tgl
@@ -765,3 +775,28 @@ void srid_is_latlong(FunctionCallInfo fcinfo, int srid)
 }
 
 
+srs_precision srid_axis_precision(FunctionCallInfo fcinfo, int srid, int precision)
+{
+	projPJ pj1;
+	projPJ pj2;
+
+	srs_precision sp;
+	
+	sp.precision_xy = precision;
+	sp.precision_z = precision;
+	sp.precision_m = precision;
+	
+	if ( srid == SRID_UNKNOWN )
+		return sp;
+
+	if ( GetProjectionsUsingFCInfo(fcinfo, srid, srid, &pj1, &pj2) == LW_FAILURE)
+		return sp;
+
+	if ( pj_is_latlong(pj1) )
+	{
+		sp.precision_xy += 5;
+		return sp;
+	}
+	
+	return sp;
+}
