@@ -1,8 +1,32 @@
+/**********************************************************************
+ *
+ * PostGIS - Spatial Types for PostgreSQL
+ * http://postgis.net
+ *
+ * PostGIS is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * PostGIS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with PostGIS.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ **********************************************************************
+ *
+ * ^copyright^
+ *
+ **********************************************************************/
+
 #include "geography_measurement_trees.h"
 
 
 /*
-* Specific tree types include all the generic slots and 
+* Specific tree types include all the generic slots and
 * their own slots for their trees. We put the implementation
 * for the CircTreeGeomCache here because we can't shove
 * the PgSQL specific bits of the code (fcinfo) back into
@@ -10,10 +34,10 @@
 */
 typedef struct {
 	int                     type;       // <GeomCache>
-	GSERIALIZED*                geom1;      // 
-	GSERIALIZED*                geom2;      // 
-	size_t                      geom1_size; // 
-	size_t                      geom2_size; // 
+	GSERIALIZED*                geom1;      //
+	GSERIALIZED*                geom2;      //
+	size_t                      geom1_size; //
+	size_t                      geom2_size; //
 	int32                       argnum;     // </GeomCache>
 	CIRC_NODE*                  index;
 } CircTreeGeomCache;
@@ -45,7 +69,7 @@ static int
 CircTreeFreer(GeomCache* cache)
 {
 	CircTreeGeomCache* circ_cache = (CircTreeGeomCache*)cache;
-	if ( circ_cache->index ) 
+	if ( circ_cache->index )
 	{
 		circ_tree_free(circ_cache->index);
 		circ_cache->index = 0;
@@ -115,7 +139,7 @@ CircTreePIP(const CIRC_NODE* tree1, const GSERIALIZED* g1, const POINT4D* in_poi
 		{
 			POINT2D pt2d_outside; /* latlon */
 			POINT2D pt2d_inside;
-			pt2d_inside.x = in_point->x; 
+			pt2d_inside.x = in_point->x;
 			pt2d_inside.y = in_point->y;
 			/* Calculate a definitive outside point */
 			gbox_pt_outside(&gbox1, &pt2d_outside);
@@ -180,7 +204,7 @@ geography_distance_cache_tolerance(FunctionCallInfoData* fcinfo, const GSERIALIZ
 		}
 		else
 		{
-			lwerror("geography_distance_cache this cannot happen!");
+			lwpgerror("geography_distance_cache this cannot happen!");
 			return LW_FAILURE;
 		}
 		
@@ -197,7 +221,7 @@ geography_distance_cache_tolerance(FunctionCallInfoData* fcinfo, const GSERIALIZ
 		}
 		
 		circtree = lwgeom_calculate_circ_tree(lwgeom);
-		if ( geomtype == POLYGONTYPE || geomtype == MULTIPOLYGONTYPE ) 
+		if ( geomtype == POLYGONTYPE || geomtype == MULTIPOLYGONTYPE )
 		{
 			POINT2D p2d;
 			circ_tree_get_point(circtree_cached, &p2d);
@@ -234,9 +258,13 @@ int
 geography_dwithin_cache(FunctionCallInfoData* fcinfo, const GSERIALIZED* g1, const GSERIALIZED* g2, const SPHEROID* s, double tolerance, int* dwithin)
 {
 	double distance;
-	/* TODO!!! Why does the tolerance stopper in the circ_tree_distance_tree_internal arbitrarily screw up? */
-/*	if ( LW_SUCCESS == geography_distance_cache_tolerance(fcinfo, g1, g2, s, tolerance, &distance) ) */
-	if ( LW_SUCCESS == geography_distance_cache_tolerance(fcinfo, g1, g2, s, FP_TOLERANCE, &distance) ) 
+	/* Ticket #2422, difference between sphere and spheroid distance can trip up the */
+	/* threshold shortcircuit (stopping a calculation before the spheroid distance is actually */
+	/* below the threshold. Lower in the code line, we actually reduce the threshold a little to */
+	/* avoid this. */
+	/* Correct fix: propogate the spheroid information all the way to the bottom of the calculation */
+	/* so the "right thing" can be done in all cases. */
+	if ( LW_SUCCESS == geography_distance_cache_tolerance(fcinfo, g1, g2, s, tolerance, &distance) )
 	{
 		*dwithin = (distance <= (tolerance + FP_TOLERANCE) ? LW_TRUE : LW_FALSE);
 		return LW_SUCCESS;
@@ -264,7 +292,7 @@ geography_tree_distance(const GSERIALIZED* g1, const GSERIALIZED* g2, const SPHE
 	{
 		*distance = 0.0;
 	}
-	else 
+	else
 	{
 		/* Calculate tree/tree distance */
 		*distance = circ_tree_distance_tree(circ_tree1, circ_tree2, s, tolerance);
