@@ -3,15 +3,26 @@
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.net
  *
- * Wrapper around external librairies functions (GEOS/CGAL...)
+ * PostGIS is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * PostGIS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with PostGIS.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ **********************************************************************
  *
  * Copyright 2012-2013 Oslandia <infos@oslandia.com>
  *
- * This is free software; you can redistribute and/or modify it under
- * the terms of the GNU General Public Licence. See the COPYING file.
- *
  **********************************************************************/
 
+
 #include "postgres.h"
 #include "fmgr.h"
 #include "utils/guc.h" /* for custom variables */
@@ -102,88 +113,6 @@ static void lwgeom_backend_switch( const char* newvalue, void* extra )
     lwpgerror("Can't find %s geometry backend", newvalue );
 }
 
-
-#include "utils/guc.h"
-#include "utils/guc_tables.h"
-
-
-/*
- * the bare comparison function for GUC names
- */
-static int
-guc_name_compare(const char *namea, const char *nameb)
-{
-	/*
-	 * The temptation to use strcasecmp() here must be resisted, because the
-	 * array ordering has to remain stable across setlocale() calls. So, build
-	 * our own with a simple ASCII-only downcasing.
-	 */
-	while (*namea && *nameb)
-	{
-		char		cha = *namea++;
-		char		chb = *nameb++;
-
-		if (cha >= 'A' && cha <= 'Z')
-			cha += 'a' - 'A';
-		if (chb >= 'A' && chb <= 'Z')
-			chb += 'a' - 'A';
-		if (cha != chb)
-			return cha - chb;
-	}
-	if (*namea)
-		return 1;				/* a is longer */
-	if (*nameb)
-		return -1;				/* b is longer */
-	return 0;
-}
-
-/*
- * comparator for qsorting and bsearching guc_variables array
- */
-static int
-guc_var_compare(const void *a, const void *b)
-{
-	const struct config_generic *confa = *(struct config_generic * const *) a;
-	const struct config_generic *confb = *(struct config_generic * const *) b;
-
-	return guc_name_compare(confa->name, confb->name);
-}
-
-/* 
-* This is copied from the top half of the find_option function 
-* in postgresql's guc.c. We search the guc_variables for our option.
-* Then we make sure it's not a placeholder. Only then are we sure 
-* we have a potential conflict, of the sort experienced during an 
-* extension upgrade.
-*/
-static int
-guc_find_option(const char *name)
-{
-	const char **key = &name;
-	struct config_generic **res;
-
-	/*
-	 * By equating const char ** with struct config_generic *, we are assuming
-	 * the name field is first in config_generic.
-	 */
-	res = (struct config_generic **) bsearch((void *) &key,
-											 (void *) get_guc_variables(),
-											 GetNumConfigOptions(),
-											 sizeof(struct config_generic *),
-											 guc_var_compare);
-	
-	/* Found nothing? Good */
-	if ( ! res ) return 0;
-	 
-	/* Hm, you found something, but maybe it's just a placeholder? */
-	/* We'll consider a placehold a "not found" */
-	if ( (*res)->flags & GUC_CUSTOM_PLACEHOLDER )
-		return 0;
-		
-	return 1;
-}
-
-
 void lwgeom_init_backend()
 {
 	/* #2382 Before trying to create a user GUC, make sure */
@@ -199,7 +128,7 @@ void lwgeom_init_backend()
 	/* callback to change which backend is in use by flipping a global variable */
 	/* over. This saves the overhead of looking up the engine every time, at */
 	/* the expense of the extra complexity. */
-	if ( guc_find_option(guc_name) )
+	if ( postgis_guc_find_option(guc_name) )
 	{
 		/* In this narrow case the previously installed GUC is tied to the callback in */
 		/* the previously loaded library. Probably this is happening during an */
@@ -303,7 +232,7 @@ Datum intersects3d_dwithin(PG_FUNCTION_ARGS)
 	error_if_srid_mismatch(lwgeom1->srid, lwgeom2->srid);
 	
     mindist = lwgeom_mindistance3d_tolerance(lwgeom1,lwgeom2,0.0);
-    
+
     PG_FREE_IF_COPY(geom1, 0);
     PG_FREE_IF_COPY(geom2, 1);
     /*empty geometries cases should be right handled since return from underlying
