@@ -1,3 +1,27 @@
+/**********************************************************************
+ *
+ * PostGIS - Spatial Types for PostgreSQL
+ * http://postgis.net
+ *
+ * PostGIS is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * PostGIS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with PostGIS.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ **********************************************************************
+ *
+ * ^copyright^
+ *
+ **********************************************************************/
+
 #include "postgres.h"
 
 #include "../postgis_config.h"
@@ -16,10 +40,18 @@
 #include "utils/elog.h"
 #include "mb/pg_wchar.h"
 # include "lib/stringinfo.h" /* for binary input */
+#include "utils/array.h"
+#include "utils/lsyscache.h"
+#include "funcapi.h"
 
 #include "liblwgeom.h"
 #include "lwgeom_pg.h"
 #include "geography.h" /* for lwgeom_valid_typmod */
+#include "lwgeom_transform.h"
+
+#if POSTGIS_PGSQL_VERSION > 92
+#include "access/htup_details.h"
+#endif
 
 void elog_ERROR(const char* string);
 
@@ -33,6 +65,10 @@ Datum parse_WKT_lwgeom(PG_FUNCTION_ARGS);
 Datum LWGEOM_recv(PG_FUNCTION_ARGS);
 Datum LWGEOM_send(PG_FUNCTION_ARGS);
 Datum LWGEOM_to_latlon(PG_FUNCTION_ARGS);
+Datum WKBFromLWGEOM(PG_FUNCTION_ARGS);
+Datum TWKBFromLWGEOM(PG_FUNCTION_ARGS);
+Datum TWKBFromLWGEOMArray(PG_FUNCTION_ARGS);
+Datum LWGEOMFromTWKB(PG_FUNCTION_ARGS);
 
 
 /*
@@ -121,7 +157,7 @@ Datum LWGEOM_in(PG_FUNCTION_ARGS)
 
 	if ( geom_typmod >= 0 )
 	{
-		postgis_valid_typmod(ret, geom_typmod);
+		ret = postgis_valid_typmod(ret, geom_typmod);
 		POSTGIS_DEBUG(3, "typmod and geometry were consistent");
 	}
 	else
@@ -162,7 +198,7 @@ PG_FUNCTION_INFO_V1(LWGEOM_to_latlon);
 Datum LWGEOM_to_latlon(PG_FUNCTION_ARGS)
 {
 	/* Get the parameters */
-	GSERIALIZED *pg_lwgeom = (GSERIALIZED *) PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *pg_lwgeom = PG_GETARG_GSERIALIZED_P(0);
 	text *format_text = PG_GETARG_TEXT_P(1);
 
 	LWGEOM *lwgeom;
@@ -176,13 +212,13 @@ Datum LWGEOM_to_latlon(PG_FUNCTION_ARGS)
 	uint8_t geom_type = gserialized_get_type(pg_lwgeom);
 	if (POINTTYPE != geom_type)
 	{
-		lwerror("Only points are supported, you tried type %s.", lwtype_name(geom_type));
+		lwpgerror("Only points are supported, you tried type %s.", lwtype_name(geom_type));
 	}
 	/* Convert to LWGEOM type */
 	lwgeom = lwgeom_from_gserialized(pg_lwgeom);
 
   if (format_text == NULL) {
-    lwerror("ST_AsLatLonText: invalid format string (null");
+    lwpgerror("ST_AsLatLonText: invalid format string (null");
     PG_RETURN_NULL();
   }
 
@@ -231,7 +267,7 @@ Datum LWGEOM_to_latlon(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_out);
 Datum LWGEOM_out(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
 	LWGEOM *lwgeom;
 	char *hexwkb;
 	size_t hexwkb_size;
@@ -249,7 +285,7 @@ Datum LWGEOM_out(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_asHEXEWKB);
 Datum LWGEOM_asHEXEWKB(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
 	LWGEOM *lwgeom;
 	char *hexwkb;
 	size_t hexwkb_size;
@@ -302,7 +338,7 @@ Datum LWGEOM_asHEXEWKB(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_to_text);
 Datum LWGEOM_to_text(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
 	LWGEOM *lwgeom;
 	char *hexwkb;
 	size_t hexwkb_size;
@@ -323,14 +359,14 @@ Datum LWGEOM_to_text(PG_FUNCTION_ARGS)
 }
 
 /*
- * LWGEOMFromWKB(wkb,  [SRID] )
+ * LWGEOMFromEWKB(wkb,  [SRID] )
  * NOTE: wkb is in *binary* not hex form.
  *
  * NOTE: this function parses EWKB (extended form)
- *       which also contains SRID info. 
+ *       which also contains SRID info.
  */
-PG_FUNCTION_INFO_V1(LWGEOMFromWKB);
-Datum LWGEOMFromWKB(PG_FUNCTION_ARGS)
+PG_FUNCTION_INFO_V1(LWGEOMFromEWKB);
+Datum LWGEOMFromEWKB(PG_FUNCTION_ARGS)
 {
 	bytea *bytea_wkb = (bytea*)PG_GETARG_BYTEA_P(0);
 	int32 srid = 0;
@@ -354,6 +390,29 @@ Datum LWGEOMFromWKB(PG_FUNCTION_ARGS)
 	PG_FREE_IF_COPY(bytea_wkb, 0);
 	PG_RETURN_POINTER(geom);
 }
+/*
+ * LWGEOMFromTWKB(wkb)
+ * NOTE: twkb is in *binary* not hex form.
+ *
+ */
+PG_FUNCTION_INFO_V1(LWGEOMFromTWKB);
+Datum LWGEOMFromTWKB(PG_FUNCTION_ARGS)
+{
+	bytea *bytea_twkb = (bytea*)PG_GETARG_BYTEA_P(0);
+	GSERIALIZED *geom;
+	LWGEOM *lwgeom;
+	uint8_t *twkb = (uint8_t*)VARDATA(bytea_twkb);
+	
+	lwgeom = lwgeom_from_twkb(twkb, VARSIZE(bytea_twkb)-VARHDRSZ, LW_PARSER_CHECK_ALL);
+
+	if ( lwgeom_needs_bbox(lwgeom) )
+		lwgeom_add_bbox(lwgeom);
+
+	geom = geometry_serialize(lwgeom);
+	lwgeom_free(lwgeom);
+	PG_FREE_IF_COPY(bytea_twkb, 0);
+	PG_RETURN_POINTER(geom);
+}
 
 /*
  * WKBFromLWGEOM(lwgeom) --> wkb
@@ -362,14 +421,13 @@ Datum LWGEOMFromWKB(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(WKBFromLWGEOM);
 Datum WKBFromLWGEOM(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = (GSERIALIZED*)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
 	LWGEOM *lwgeom;
 	uint8_t *wkb;
 	size_t wkb_size;
 	uint8_t variant = 0;
  	bytea *result;
 	text *type;
-
 	/* If user specified endianness, respect it */
 	if ( (PG_NARGS()>1) && (!PG_ARGISNULL(1)) )
 	{
@@ -385,9 +443,10 @@ Datum WKBFromLWGEOM(PG_FUNCTION_ARGS)
 			variant = variant | WKB_NDR;
 		}
 	}
-
+	wkb_size= VARSIZE(geom) - VARHDRSZ;
 	/* Create WKB hex string */
 	lwgeom = lwgeom_from_gserialized(geom);
+
 	wkb = lwgeom_to_wkb(lwgeom, variant | WKB_EXTENDED , &wkb_size);
 	lwgeom_free(lwgeom);
 	
@@ -402,12 +461,234 @@ Datum WKBFromLWGEOM(PG_FUNCTION_ARGS)
 	PG_RETURN_BYTEA_P(result);
 }
 
+PG_FUNCTION_INFO_V1(TWKBFromLWGEOM);
+Datum TWKBFromLWGEOM(PG_FUNCTION_ARGS)
+{
+	GSERIALIZED *geom;
+	LWGEOM *lwgeom;
+	uint8_t *twkb;
+	size_t twkb_size;
+	uint8_t variant = 0;
+ 	bytea *result;
+	srs_precision sp;
+	
+	/*check for null input since we cannot have the sql-function as strict.
+	That is because we use null as default for optional ID*/	
+	if ( PG_ARGISNULL(0) ) PG_RETURN_NULL();
+	
+	geom = PG_GETARG_GSERIALIZED_P(0);
+
+	/* Read sensible precision defaults (about one meter) given the srs */
+	sp = srid_axis_precision(fcinfo, gserialized_get_srid(geom), TWKB_DEFAULT_PRECISION);
+	
+	/* If user specified XY precision, use it */
+	if ( PG_NARGS() > 1 && ! PG_ARGISNULL(1) )
+		sp.precision_xy = PG_GETARG_INT32(1);
+
+	/* If user specified Z precision, use it */
+	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
+		sp.precision_z = PG_GETARG_INT32(2);
+
+	/* If user specified M precision, use it */
+	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
+		sp.precision_m = PG_GETARG_INT32(3);
+
+	/* We don't permit ids for single geoemtries */
+	variant = variant & ~TWKB_ID;
+
+	/* If user wants registered twkb sizes */
+	if ( PG_NARGS() > 4 && ! PG_ARGISNULL(4) && PG_GETARG_BOOL(4) )
+		variant |= TWKB_SIZE;
+	
+	/* If user wants bounding boxes */
+	if ( PG_NARGS() > 5 && ! PG_ARGISNULL(5) && PG_GETARG_BOOL(5) )
+		variant |= TWKB_BBOX;
+
+	/* Create TWKB binary string */
+	lwgeom = lwgeom_from_gserialized(geom);
+	twkb = lwgeom_to_twkb(lwgeom, variant, sp.precision_xy, sp.precision_z, sp.precision_m, &twkb_size);
+	lwgeom_free(lwgeom);
+	
+	/* Prepare the PgSQL text return type */
+	result = palloc(twkb_size + VARHDRSZ);
+	memcpy(VARDATA(result), twkb, twkb_size);
+	SET_VARSIZE(result, twkb_size + VARHDRSZ);
+	
+	pfree(twkb);
+	PG_FREE_IF_COPY(geom, 0);
+	PG_RETURN_BYTEA_P(result);
+}
+
+
+PG_FUNCTION_INFO_V1(TWKBFromLWGEOMArray);
+Datum TWKBFromLWGEOMArray(PG_FUNCTION_ARGS)
+{
+	ArrayType *arr_geoms = NULL;
+	ArrayType *arr_ids = NULL;
+	int num_geoms, num_ids, i = 0;
+
+	ArrayIterator iter_geoms, iter_ids;
+	bool null_geom, null_id;
+	Datum val_geom, val_id;
+
+	int is_homogeneous = true;
+	int subtype = 0;
+	int has_z = 0;
+	int has_m  = 0;
+	LWCOLLECTION *col = NULL;
+	int64_t *idlist = NULL;
+	uint8_t variant = 0;
+
+	srs_precision sp;
+	uint8_t *twkb;
+	size_t twkb_size;
+ 	bytea *result;
+
+	/* The first two arguments are required */
+	if ( PG_NARGS() < 2 || PG_ARGISNULL(0) || PG_ARGISNULL(1) )
+		PG_RETURN_NULL();
+
+	arr_geoms = PG_GETARG_ARRAYTYPE_P(0);
+	arr_ids = PG_GETARG_ARRAYTYPE_P(1);
+
+	num_geoms = ArrayGetNItems(ARR_NDIM(arr_geoms), ARR_DIMS(arr_geoms));
+	num_ids = ArrayGetNItems(ARR_NDIM(arr_ids), ARR_DIMS(arr_ids));
+	
+	if ( num_geoms != num_ids )
+	{
+		elog(ERROR, "size of geometry[] and integer[] arrays must match");
+		PG_RETURN_NULL();
+	}
+
+	/* Loop through array and build a collection of geometry and */
+	/* a simple array of ids. If either side is NULL, skip it */
+
+#if POSTGIS_PGSQL_VERSION >= 95	
+	iter_geoms = array_create_iterator(arr_geoms, 0, NULL);
+	iter_ids = array_create_iterator(arr_ids, 0, NULL);
+#else
+	iter_geoms = array_create_iterator(arr_geoms, 0);
+	iter_ids = array_create_iterator(arr_ids, 0);
+#endif
+
+	while( array_iterate(iter_geoms, &val_geom, &null_geom) &&
+	       array_iterate(iter_ids, &val_id, &null_id) )
+	{
+		LWGEOM *geom;
+		int32_t uid;
+
+		if ( null_geom || null_id )
+		{
+			elog(NOTICE, "ST_AsTWKB skipping NULL entry at position %d", i);
+			continue;
+		}
+
+		geom = lwgeom_from_gserialized((GSERIALIZED*)DatumGetPointer(val_geom));
+		uid = DatumGetInt64(val_id);
+		
+		/* Construct collection/idlist first time through */
+		if ( ! col )
+		{
+			has_z = lwgeom_has_z(geom);
+			has_m = lwgeom_has_m(geom);
+			col = lwcollection_construct_empty(COLLECTIONTYPE, lwgeom_get_srid(geom), has_z, has_m);
+		}
+		if ( ! idlist )
+			idlist = palloc0(num_geoms * sizeof(int64_t));
+
+		
+		/*Check if there is differences in dimmenstionality*/
+		if( lwgeom_has_z(geom)!=has_z || lwgeom_has_m(geom)!=has_m)
+		{
+			elog(ERROR, "Geometries have differenct dimensionality");
+			PG_FREE_IF_COPY(arr_geoms, 0);
+			PG_FREE_IF_COPY(arr_ids, 1);
+			PG_RETURN_NULL();			
+		}
+		/* Store the values */
+		lwcollection_add_lwgeom(col, geom);
+		idlist[i++] = uid;
+		
+		/* Grab the geometry type and note if all geometries share it */
+		/* If so, we can make this a homogeneous collection and save some space */
+		if ( lwgeom_get_type(geom) != subtype && subtype )
+		{
+			is_homogeneous = false;
+		}
+		else
+		{
+			subtype = lwgeom_get_type(geom);
+		}
+
+	}
+	array_free_iterator(iter_geoms);
+	array_free_iterator(iter_ids);
+	
+	if(i==0)
+	{
+		elog(NOTICE, "No valid geometry - id pairs found");
+		PG_FREE_IF_COPY(arr_geoms, 0);
+		PG_FREE_IF_COPY(arr_ids, 1);
+		PG_RETURN_NULL();		
+	}
+	if ( is_homogeneous )
+	{
+		col->type = lwtype_get_collectiontype(subtype);
+	}
+
+	/* Read sensible precision defaults (about one meter) given the srs */
+	sp = srid_axis_precision(fcinfo, lwgeom_get_srid(lwcollection_as_lwgeom(col)), TWKB_DEFAULT_PRECISION);
+	
+	/* If user specified XY precision, use it */
+	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
+		sp.precision_xy = PG_GETARG_INT32(2);
+
+	/* If user specified Z precision, use it */
+	if ( PG_NARGS() > 3 && ! PG_ARGISNULL(3) )
+		sp.precision_z = PG_GETARG_INT32(3);
+
+	/* If user specified M precision, use it */
+	if ( PG_NARGS() > 4 && ! PG_ARGISNULL(4) )
+		sp.precision_m = PG_GETARG_INT32(4);
+
+	/* We are building an ID'ed output */
+	variant = TWKB_ID;
+	
+	/* If user wants registered twkb sizes */
+	if ( PG_NARGS() > 5 && ! PG_ARGISNULL(5) && PG_GETARG_BOOL(5) )
+		variant |= TWKB_SIZE;
+	
+	/* If user wants bounding boxes */
+	if ( PG_NARGS() > 6 && ! PG_ARGISNULL(6) && PG_GETARG_BOOL(6) )
+		variant |= TWKB_BBOX;
+
+	/* Write out the TWKB */
+	twkb = lwgeom_to_twkb_with_idlist(lwcollection_as_lwgeom(col),
+	                                  idlist, variant,
+	                                  sp.precision_xy, sp.precision_z, sp.precision_m,
+	                                  &twkb_size);
+					
+	/* Convert to a bytea return type */
+	result = palloc(twkb_size + VARHDRSZ);
+	memcpy(VARDATA(result), twkb, twkb_size);
+	SET_VARSIZE(result, twkb_size + VARHDRSZ);
+	
+	/* Clean up */
+	pfree(twkb);
+	pfree(idlist);
+	lwcollection_free(col);
+	PG_FREE_IF_COPY(arr_geoms, 0);
+	PG_FREE_IF_COPY(arr_ids, 1);
+	
+	PG_RETURN_BYTEA_P(result);
+}
+
 
 /* puts a bbox inside the geometry */
 PG_FUNCTION_INFO_V1(LWGEOM_addBBOX);
 Datum LWGEOM_addBBOX(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
 	GSERIALIZED *result;
 	LWGEOM *lwgeom;
 
@@ -423,7 +704,7 @@ Datum LWGEOM_addBBOX(PG_FUNCTION_ARGS)
 PG_FUNCTION_INFO_V1(LWGEOM_dropBBOX);
 Datum LWGEOM_dropBBOX(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *geom = (GSERIALIZED *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	GSERIALIZED *geom = PG_GETARG_GSERIALIZED_P(0);
 
 	/* No box? we're done already! */
 	if ( ! gserialized_has_bbox(geom) )
@@ -453,13 +734,13 @@ Datum parse_WKT_lwgeom(PG_FUNCTION_ARGS)
 	Datum result;
 
 	/* Unwrap the PgSQL text type into a cstring */
-	wkt = text2cstring(wkt_text); 
+	wkt = text2cstring(wkt_text);
 	
 	/* Now we call over to the geometry_in function */
 	result = DirectFunctionCall1(LWGEOM_in, CStringGetDatum(wkt));
 
 	/* Return null on null */
-	if ( ! result ) 
+	if ( ! result )
 		PG_RETURN_NULL();
 
 	PG_RETURN_DATUM(result);
@@ -498,7 +779,7 @@ Datum LWGEOM_recv(PG_FUNCTION_ARGS)
 
 	if ( geom_typmod >= 0 )
 	{
-		postgis_valid_typmod(geom, geom_typmod);
+		geom = postgis_valid_typmod(geom, geom_typmod);
 		POSTGIS_DEBUG(3, "typmod and geometry were consistent");
 	}
 	else
@@ -520,7 +801,7 @@ Datum LWGEOM_send(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(
 	  DatumGetPointer(
 	    DirectFunctionCall1(
-	      WKBFromLWGEOM, 
+	      WKBFromLWGEOM,
 	      PG_GETARG_DATUM(0)
 	    )));
 }
@@ -533,7 +814,7 @@ Datum LWGEOM_to_bytea(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(
 	  DatumGetPointer(
 	    DirectFunctionCall1(
-	      WKBFromLWGEOM, 
+	      WKBFromLWGEOM,
 	      PG_GETARG_DATUM(0)
 	    )));
 }
@@ -546,7 +827,7 @@ Datum LWGEOM_from_bytea(PG_FUNCTION_ARGS)
 	POSTGIS_DEBUG(2, "LWGEOM_from_bytea start");
 
 	result = (GSERIALIZED *)DatumGetPointer(DirectFunctionCall1(
-	                                          LWGEOMFromWKB, PG_GETARG_DATUM(0)));
+	                                          LWGEOMFromEWKB, PG_GETARG_DATUM(0)));
 
 	PG_RETURN_POINTER(result);
 }
