@@ -1,9 +1,10 @@
-#!/usr/bin/perl
+#!/usr/bin/perl -w
 
 #
 # PostGIS - Spatial Types for PostgreSQL
-# http://postgis.refractions.net
+# http://postgis.net
 #
+# Copyright (C) 2014 Sandro Santilli <strk@kbt.io>
 # Copyright (C) 2009-2010 Paul Ramsey <pramsey@opengeo.org>
 # Copyright (C) 2005 Refractions Research Inc.
 #
@@ -52,32 +53,10 @@ my $objs = {
 			"st_makeline(geometry)" => 1
 		}
 	},
- 	"104" => { 
-		"types" => {
-			"box3d_extent" => 1,
-			"pgis_abs" => 1
-		}
-	},
  	"105" => { 
-		"operators" => {
-			"geography >" => 1,
-			"geography >=" => 1,
-			"geography =" => 1,
-			"geography <=" => 1,
-			"geography <" => 1,
-			"geography &&" => 1 
-		},
-		"opclasses" => {
-			"gist_geography_ops" => 1,
-			"btree_geography_ops" => 1
-		},
 		"views" => {
 			"geography_columns" => 1
 		},
-		"types" => {
-			"geography" => 1,
-			"gidx" => 1
-		}
 	},
  	"200" => { 
 		"aggregates" => {
@@ -93,16 +72,52 @@ my $objs = {
 			"st_union(raster,integer)" => 1,
 			"st_union(raster)" => 1,
 			"st_union(raster,text)" => 1
-		},
-		"operators" => {
-			"raster =" => 1
-		},
-		"opclasses" => {
-			"hash_raster_ops" => 1
-		},
+		}
+	},
+ 	"202" => { 
+		"aggregates" => {
+			"st_summarystatsagg(raster,integer,boolean,double precision)" => 1,
+			"st_summarystatsagg(raster,boolean,double precision)" => 1,
+			"st_summarystatsagg(raster,integer)" => 1,
+			"st_countagg(raster,integer,boolean,double precision)" => 1,
+			"st_countagg(raster,integer,boolean)" => 1,
+			"st_countagg(raster,boolean)" => 1
+		}
 	}
 };
 
+sub find_last_updated
+{
+  my $type = shift;
+  my $sig = shift;
+  for my $ver ( sort { $b cmp $a } keys %$objs ) {
+    if ( $objs->{$ver}->{$type}->{$sig} ) {
+      return $ver;
+    }
+  }
+  return 0;
+}
+
+sub parse_last_updated
+{
+  my $comment = shift;
+  if ( $comment =~ m/.*(?:Availability|Changed|Updated):\s([^\.])\.([^.]*)/s ) {
+    return $1*100 + $2;
+  }
+  return 0;
+}
+
+sub parse_missing
+{
+  my $comment = shift;
+  my @missing = ();
+  if ( $comment =~ m/.*(?:Missing in):\s([^\.])\.([^.]*)/s ) {
+    push(@missing, $1*100 + $2);
+  }
+  return join(',',@missing);
+}
+
+
 #
 # Commandline argument handling
 #
@@ -112,6 +127,7 @@ die "Usage: perl postgis_proc_upgrade.pl <postgis.sql> <version_from> [<schema>]
 
 my $sql_file = $ARGV[0];
 my $module = 'postgis';
+my $soname = '';
 my $version_to = "";
 my $version_to_num = 0;
 my $version_from = $ARGV[1];
@@ -119,15 +135,6 @@ my $version_from_num = 0;
 my $schema = "";
 $schema = $ARGV[2] if @ARGV > 2;
 
-if ( $version_from =~ /^(\d+)\.(\d+)/ )
-{
-	$version_from_num = 100 * $1 + $2; 
-}
-else
-{
-	die "Version from number invalid, must be of form X.X\n";
-}
-
 die "Unable to open input SQL file $sql_file\n"
 	if ( ! -f $sql_file );
 
@@ -144,12 +151,16 @@ while(<INPUT>)
 	if (/INSTALL VERSION: (.*)/)
 	{
 				$version_to = $1;
-				last;
+				#last;
 	}
 	elsif (/TYPE raster/)
 	{
         $module = 'postgis_raster';
 	}
+	elsif (m@('\$libdir/[^']*')@)
+	{
+        $soname = $1;
+	}
 }
 close(INPUT); 
 
@@ -168,12 +179,14 @@ else
 
 print qq{
 --
--- UPGRADE SCRIPT FROM PostGIS $version_from TO PostGIS $version_to
+-- UPGRADE SCRIPT TO PostGIS $version_to
 --
 
 };
 
-print "BEGIN;\n";
+print "LOAD $soname;\n" if ($soname);
+
+#print "BEGIN;\n";
 print "SET search_path TO $schema;\n" if $schema;
 
 #
@@ -192,17 +205,21 @@ while(<DATA>)
 # applied to an existing, loaded database: types and operators
 # and operator classes that have already been defined.
 #
+my $comment = '';
 open( INPUT, $sql_file ) || die "Couldn't open file: $sql_file\n";
 while(<INPUT>)
 {
 
-	next if ( /^\-\-/ );
+	if ( /^\-\-/ ) {
+		$comment .= $_;
+		next;
+	}
 
 	#
 	# Allow through deprecations from postgis_drop.sql
 	#
-	print if ( /^drop function if exists/i );
-	print if ( /^drop aggregate if exists/i );
+	print if ( /^drop function /i );
+	print if ( /^drop aggregate /i );
 
 	if ( /^create or replace function/i )
 	{
@@ -225,16 +242,29 @@ while(<INPUT>)
 			$def .= $_;
 			last if /\)/;
 		}
-		my $ver = $version_from_num + 1;
-		while( $version_from_num < $version_to_num && $ver <= $version_to_num )
-		{
-			if( $objs->{$ver}->{"types"}->{$newtype} )
-			{
-				print $def;
-				last;
-			}
-			$ver++;
-		}
+
+    my $last_updated = parse_last_updated($comment);
+    if ( ! $last_updated ) {
+      print STDERR "WARNING: no last updated info for type '${newtype}'\n";
+      $last_updated = find_last_updated("types", $newtype);
+    }
+    my $missing = parse_missing($comment);
+    print "-- Type ${newtype} -- LastUpdated: ${last_updated}\n";
+      print <<"EOF";
+DO LANGUAGE 'plpgsql'
+\$postgis_proc_upgrade\$
+BEGIN
+  IF $last_updated > version_from_num
+EOF
+      print "OR version_from_num IN ( ${missing} )" if ( $missing );
+      print <<"EOF";
+     FROM _postgis_upgrade_info
+  THEN
+      EXECUTE \$postgis_proc_upgrade_parsed_def\$ $def \$postgis_proc_upgrade_parsed_def\$;
+  END IF;
+END
+\$postgis_proc_upgrade\$;
+EOF
 	}
 
 	if ( /^do *language .*\$\$/i )
@@ -282,17 +312,28 @@ while(<INPUT>)
 			last if /\);/;
 		}
 		my $aggsig = "$aggname($aggtype)";
-		my $ver = $version_from_num + 1;
-    #print "-- Checking ${aggsig} -- From: ${version_from_num} -- To: ${version_to_num}\n";
-		while( $version_from_num < $version_to_num && $ver <= $version_to_num )
-		{
-			if( $objs->{$ver}->{"aggregates"}->{$aggsig} )
-			{
-        print "DROP AGGREGATE IF EXISTS $aggsig;\n";
-        print $def;
-			}
-			$ver++;
-		}
+
+    #print "-- Checking comment $comment\n";
+    my $last_updated = parse_last_updated($comment);
+    if ( ! $last_updated ) {
+      print STDERR "WARNING: no last updated info for aggregate '${aggsig}'\n";
+      $last_updated = find_last_updated("aggregates", $aggsig);
+    }
+    print "-- Aggregate ${aggsig} -- LastUpdated: ${last_updated}\n";
+      print <<"EOF";
+DO LANGUAGE 'plpgsql'
+\$postgis_proc_upgrade\$
+BEGIN
+  IF $last_updated > version_from_num OR (
+      $last_updated = version_from_num AND version_from_isdev
+    ) FROM _postgis_upgrade_info
+  THEN
+    EXECUTE 'DROP AGGREGATE IF EXISTS $aggsig';
+    EXECUTE \$postgis_proc_upgrade_parsed_def\$ $def \$postgis_proc_upgrade_parsed_def\$;
+  END IF;
+END
+\$postgis_proc_upgrade\$;
+EOF
 	}
 	
 	# This code handles operators by creating them if we are doing a major upgrade
@@ -308,16 +349,23 @@ while(<INPUT>)
 			last if /\);/;
 		}
 		my $opsig = $optype . " " . $opname;
-		my $ver = $version_from_num + 1;
-		while( $version_from_num < $version_to_num && $ver <= $version_to_num )
-		{
-			if( $objs->{$ver}->{"operators"}->{$opsig} )
-			{
-				print $def;
-				last;
-			}
-			$ver++;
-		}
+
+    my $last_updated = parse_last_updated($comment);
+    if ( ! $last_updated ) {
+      print STDERR "WARNING: no last updated info for operator '${opsig}'\n";
+      $last_updated = find_last_updated("operators", $opsig);
+    }
+    print "-- Operator ${opsig} -- LastUpdated: ${last_updated}\n";
+      print <<"EOF";
+DO LANGUAGE 'plpgsql'
+\$postgis_proc_upgrade\$
+BEGIN
+  IF $last_updated > version_from_num FROM _postgis_upgrade_info THEN
+    EXECUTE \$postgis_proc_upgrade_parsed_def\$ $def \$postgis_proc_upgrade_parsed_def\$;
+  END IF;
+END
+\$postgis_proc_upgrade\$;
+EOF
 	}
 
 	# Always output create ore replace view (see ticket #1097)
@@ -342,6 +390,37 @@ while(<INPUT>)
 		}
 	}
 
+	# This code handles operator family by creating them if we are doing a major upgrade
+	if ( /^create operator family\s+(\w+)\s+USING\s+(\w+)\s*/i )
+	{
+		my $opfname = $1;
+		my $amname = $2;
+		my $def = $_;
+		my $opfsig = $opfname . " " . $amname;
+		while(<INPUT>)
+		{
+			$def .= $_;
+			last if /\);/;
+		}
+
+	my $last_updated = parse_last_updated($comment);
+	if ( ! $last_updated ) {
+		print STDERR "WARNING: no last updated info for operator family '${opfname}'\n";
+		$last_updated = find_last_updated("opfamilies", $opfsig);
+	}
+	print "-- Operator family ${opfsig} -- LastUpdated: ${last_updated}\n";
+	print <<"EOF";
+DO LANGUAGE 'plpgsql'
+\$postgis_proc_upgrade\$
+BEGIN
+  IF $last_updated > version_from_num FROM _postgis_upgrade_info THEN
+    EXECUTE \$postgis_proc_upgrade_parsed_def\$ $def \$postgis_proc_upgrade_parsed_def\$;
+  END IF;
+END
+\$postgis_proc_upgrade\$;
+EOF
+	}
+
 	# This code handles operator classes by creating them if we are doing a major upgrade
 	if ( /^create operator class\s+(\w+)\s*/i )
 	{
@@ -349,31 +428,93 @@ while(<INPUT>)
 		my $opctype = 'unknown';
 		my $opcidx = 'unknown';
 		my $def = $_;
+		my $last_updated;
+		my $subcomment = '';
+		my @subobjects; # minversion, definition
 		while(<INPUT>)
 		{
+			if ( /^\s*\-\-/ ) {
+				$subcomment .= $_;
+				next;
+			}
+
 			$def .= $_;
 			$opctype = $1 if ( /for type (\w+) /i );
 			$opcidx = $1 if ( /using (\w+) /i );
+
+			# Support adding members at later versions
+			if ( /\s+(OPERATOR|FUNCTION)\s+[0-9]+\s+ / )
+			{
+				my $last_updated = parse_last_updated($subcomment);
+				if ( $last_updated )
+				{
+					my $subdefn = $_;
+					chop $subdefn;
+					$subdefn =~ s/[,;]$//; # strip ending comma or semicolon
+					# argument types must be specified in ALTER OPERATOR FAMILY
+					if ( $subdefn =~ m/\s+(OPERATOR.*)(FOR.*)/ )
+					{
+						$subdefn = $1.'('.$opctype.','.$opctype.') '.$2;
+					}
+					elsif ( $subdefn =~ m/\s+(FUNCTION\s+[0-9]+ )(.*)/ )
+					{
+						$subdefn = $1.'('.$opctype.','.$opctype.') '.$2;
+					}
+					push @subobjects, [$last_updated, $subdefn];
+				}
+				$subcomment = '';
+			}
 			last if /\);/;
 		}
 		$opctype =~ tr/A-Z/a-z/;
 		$opcidx =~ tr/A-Z/a-z/;
-		my $ver = $version_from_num + 1;
-		while( $version_from_num < $version_to_num && $ver <= $version_to_num )
-		{
-			if( $objs->{$ver}->{"opclasses"}->{$opclassname} )
-			{
-				print $def;
-				last;
-			}
-			$ver++;
-		}
+
+    $last_updated = parse_last_updated($comment);
+    if ( ! $last_updated ) {
+      print STDERR "WARNING: no last updated info for operator class '${opclassname}'\n";
+      $last_updated = find_last_updated("opclasses", $opclassname);
+    }
+    print "-- Operator class ${opclassname} -- LastUpdated: ${last_updated}\n";
+    print <<"EOF";
+DO LANGUAGE 'plpgsql'
+\$postgis_proc_upgrade\$
+BEGIN
+  IF $last_updated > version_from_num FROM _postgis_upgrade_info THEN
+    EXECUTE \$postgis_proc_upgrade_parsed_def\$
+    $def    \$postgis_proc_upgrade_parsed_def\$;
+EOF
+    my $ELSE="ELSE -- version_from >= $last_updated";
+    for my $subobj ( @subobjects )
+    {
+      $last_updated = @{$subobj}[0];
+      $def = @{$subobj}[1];
+      print <<"EOF";
+  $ELSE
+    -- Last Updated: ${last_updated}
+    IF $last_updated > version_from_num FROM _postgis_upgrade_info THEN
+      EXECUTE \$postgis_proc_upgrade_parsed_def\$
+        ALTER OPERATOR FAMILY ${opclassname} USING ${opcidx}
+          ADD $def;
+      \$postgis_proc_upgrade_parsed_def\$;
+    END IF;
+EOF
+      $ELSE="";
+    }
+    print <<"EOF";
+  END IF; -- version_from >= $last_updated
+END
+\$postgis_proc_upgrade\$;
+EOF
 	}
+
+	$comment = '';
 }
 
 close( INPUT );
 
-print "COMMIT;\n";
+print "DROP TABLE _postgis_upgrade_info;\n";
+
+#print "COMMIT;\n";
 
 1;
 
@@ -390,24 +531,22 @@ DECLARE
 BEGIN
 	--
 	-- This uses postgis_lib_version() rather then
-	-- postgis_scripts_installed() as in 1.0 because
+	-- MODULE_scripts_installed() as in 1.0 because
 	-- in the 1.0 => 1.1 transition that would result
 	-- in an impossible upgrade:
 	--
 	--   from 0.3.0 to 1.1.0
 	--
 	-- Next releases will still be ok as
-	-- postgis_lib_version() and postgis_scripts_installed()
+	-- postgis_lib_version() and MODULE_scripts_installed()
 	-- would both return actual PostGIS release number.
 	-- 
-
 	BEGIN
 		SELECT into old_scripts MODULE_lib_version();
 	EXCEPTION WHEN OTHERS THEN
 		RAISE DEBUG ''Got %'', SQLERRM;
 		SELECT into old_scripts MODULE_scripts_installed();
 	END;
-
 	SELECT into new_scripts ''NEWVERSION'';
 	SELECT into old_maj substring(old_scripts from 1 for 2);
 	SELECT into new_maj substring(new_scripts from 1 for 2);
@@ -424,3 +563,21 @@ LANGUAGE 'plpgsql';
 SELECT postgis_major_version_check();
 
 DROP FUNCTION postgis_major_version_check();
+
+CREATE TEMPORARY TABLE _postgis_upgrade_info AS WITH versions AS (
+  SELECT 'NEWVERSION'::text as upgraded,
+  MODULE_scripts_installed() as installed
+) SELECT
+  upgraded as scripts_upgraded,
+  installed as scripts_installed,
+  substring(upgraded from '([0-9]*)\.')::int * 100 +
+  substring(upgraded from '[0-9]*\.([0-9]*)\.')::int
+    as version_to_num,
+  substring(installed from '([0-9]*)\.')::int * 100 +
+  substring(installed from '[0-9]*\.([0-9]*)\.')::int
+    as version_from_num,
+  position('dev' in  installed)::bool
+    as version_from_isdev
+  FROM versions
+;
+ 
